@* The gaxpy functional. Our first functional is arguably the most simple. We define 
a functional that computes the scalar |gaxpy| operation, that is, $f(x)=a^Tx+b$.
@c
/* Empty, waiting for |export| */
@ @(functional/gaxpy.h@>=
#ifndef __MATH_GAXPY_FUNCTIONAL__
#define __MATH_GAXPY_FUNCTIONAL__ 1.00
#include <math/algebra.h>
#include <math/functionalbase.h>
namespace math
{
    @[namespace functional@]
    {
	template<matrix_simple_template>@;
	class @[gaxpy : public base<T>@]
	{
	    @<Gaxpy functional internal variables@>@;
	public:@;
	    @<Gaxpy functional class methods@>@;
	};
    }
}
#endif
@ To compute a |gaxpy| operation we need two parameters, the~$a$ vector and the scalar~$b$.
@<Gaxpy functional internal variables@>=
matrix<T,structure,storage> ay;
T bee;
@ We provide a means to modify these values in two ways: by returning a reference to them
and at the time of construction.
@<Gaxpy functional class methods@>=@!@:functional::gaxpy}{A@>@!@:functional::gaxpy}{b@>
matrix<T,structure,storage>& a(void) @+ {@+return ay;@+}
T& b(void) @+ {@+return bee;@+}
gaxpy(const matrix<T,structure,storage>& A,const T& B):
    ay(A), bee(B) @+
{}
@ Now to the fun stuff. Evaluating is a simple task of calling some algebra functions.
@<Gaxpy functional class methods@>=
virtual T eval(const matrix<T,unstructured,dense>& x)
{@+
    return dot(ay,x)+bee;@+
}
@ The gradient is simply~$a$.
@<Gaxpy functional class methods@>=@!@:functional::gaxpy}{grad@>
virtual matrix<T,unstructured,dense>&
grad(const matrix<T,unstructured,dense>& x, matrix<T,unstructured,dense>& dest)
{@+
    return (dest=ay);@+
}
@ The hessian of an affine function is zero.
@<Gaxpy functional class methods@>=@!@:functional::gaxpy}{hess@>
virtual matrix<T,symmetric,dense>&
hess(const matrix<T,unstructured,dense>& x, matrix<T,symmetric,dense>& dest)
{
    dest.init(x.rows(),x.rows());
    dest.fillwith(0);
    return dest;
}
virtual matrix<T,symmetric,sparse>&
hess(const matrix<T,unstructured,dense>& x, matrix<T,symmetric,sparse>& dest)
{
    dest=@[matrix<T,symmetric,sparse>@](x.rows(),x.rows());
    return dest;
}
