@* The power functional.%%
@c
/* Empty, waiting for |export| */
@ @(functional/power.h@>=
#ifndef __MATH_POWER_FUNCTIONAL__
#define __MATH_POWER_FUNCTIONAL__ 1.00
#include <math.h>
#include <math/functionalbase.h>
namespace math
{
    @[namespace functional@]
    {
	template<class T>@;
	class @[power : public base<T>@]
	{
	    @<Power of a functional internal variables@>@;
	public:@;
	    @<Power of a functional class methods@>@;
	};
    }
}
#endif
@ 
@<Power of a functional internal variables@>=
typedef base<T>* element_type;
element_type f;
double exponent;
@ 
@<Power of a functional class methods@>=
power(element_type new_f,double new_e) @[:@] f(new_f),exponent(new_e) @+{@+}
@ 
@<Power of a functional class methods@>=
T eval(const matrix<T,unstructured,dense>& x)
{
    T result=f->eval(x);
    return pow(result,exponent);
}
@ The gradient of the power is given by $nf^{n-1}\nabla f$.
@ @<Power of a functional class methods@>=
matrix<T,unstructured,dense>& grad(const matrix<T,unstructured,dense>& x,
				   matrix<T,unstructured,dense>& dest)
{
    T result=f->eval(x);
    result=exponent*pow(result,exponent-1);
    f->grad(x,dest);
    return dest*=result;
}
@ The Hessian is given by $nf^{n-1}\nabla^2f+n(n-1)f^{n-2}\nabla f\nabla f^T$.
@<Power of a functional internal variables@>=
matrix<T,symmetric,dense> Hdense;
matrix<T,symmetric,sparse> Hsparse;
matrix<T,unstructured,dense> aux;
@ @<Power of a functional class methods@>=
matrix<T,symmetric,dense> &hess(const matrix<T,unstructured,dense>&x, matrix<T,symmetric,dense>&
				dest)
{
    T result=f->eval(x);
    if(result==0 && exponent>=2)
    {
        dest.resize(x.rows(),x.rows());
	dest.fillwith(0.0);
	return dest;
    }
    if(result==0 && exponent<2) throw error::domain();
    f->grad(x,aux);
    outerp(aux,&Hdense);
    Hdense*=(exponent-1)/result;
    f->hess(x,dest);
    dest+=Hdense;
    return dest*=pow(result,exponent-1)*exponent;
}
matrix<T,symmetric,sparse> &hess(const matrix<T,unstructured,dense>&x, matrix<T,symmetric,sparse>&
				dest)
{
    T result=f->eval(x);
    if(result==0 && exponent>=2)
    {
        dest.resize(x.rows(),x.rows());
	dest.fillwith(0.0);
	return dest;
    }
    if(result==0 && exponent<2) throw error::domain();
    f->grad(x,aux);
    outerp(aux,&Hsparse);
    Hsparse*=(exponent-1)/result;
    f->hess(x,dest);
    dest+=Hsparse;
    return dest*=pow(result,exponent-1)*exponent;
}
