@* The quadratic functional. The next functional we implement is the quadratic. It computes the
value of~$f(x)=x^TPx+p^Tx+\pi$. We will require~$P$ to be symmetric. If it is not, it is always
possible to find a new~$P$ that results in the same functional. To the code:
@c
/* Empty, waiting for |export| */
@ @(functional/quadratic.h@>=
#ifndef __MATH_QUADRATIC_FUNCTIONAL__
#define __MATH_QUADRATIC_FUNCTIONAL__ 1.00
#include <math/algebra.h>
#include <math/functionalbase.h>
namespace math
{
    @[namespace functional@]
    {
	template <class T,template <@[class@]> class @[storage=dense@]>@;
	class @[quadratic : public base<T>@]
	{
	    @<Quadratic functional internal variables@>@;
	public:@;
	    @<Quadratic functional methods@>@;
	};
    }
}
#endif
@ We first provide storage for the functional parameters. As a design decision, the vector~$p$
will be dense and unstructured.
@<Quadratic functional internal variables@>=
matrix<T,symmetric,storage> Pee;
matrix<T,unstructured,dense> pee;
T Pi;
@ We provide means to change the parameters in the same way we did with the |gaxpy| functional.
@<Quadratic functional methods@>=
matrix<T,symmetric,storage>& P(void)@+{@+return Pee;@+}
matrix<T,unstructured,dense>& p(void)@+{@+return pee;@+}
T& pi(void)@+{@+return Pi;@+}
quadratic(const matrix<T,symmetric,storage>& newP, 
	  const matrix<T,unstructured,dense>& newp,
	  const T& newpi) : Pee(newP), pee(newp), Pi(newpi)@+
{}
@ Evaluating: we use the |axmul| function for now, but it's a good idea to change it in the
future. Probably the best thing to do is to define a good |*| operator, but I don't have one
right now.
@<Quadratic functional methods@>=@!@:functional::quadratic}{eval@>
virtual T eval(const matrix<T,unstructured,dense>& x)
{
    T result=dot(pee,x)+Pi;
    matrix<T,unstructured,dense> y(Pee.rows(),1);
    y=axmul(Pee,x,&y);
    result+=dot(x,y);
    return result;
}
@ The gradient is given by~$2Px+p$.
@<Quadratic functional methods@>=@!@:functional::quadratic}{grad@>
virtual matrix<T,unstructured,dense>&
grad(const matrix<T,unstructured,dense>& x, matrix<T,unstructured,dense>& dest)
{
    return gaxpy(Pee,x*2,&(dest.copyfrom(pee)));
}
@ And the Hessian is simply~$2P$.
@<Quadratic functional methods@>=@!@:functional::quadratic}{hess@>
virtual matrix<T,symmetric,dense>&
hess(const matrix<T,unstructured,dense>& x, matrix<T,symmetric,dense>& dest)
{
    dest=Pee;
    return dest*=2;
}
virtual matrix<T,symmetric,sparse>&
hess(const matrix<T,unstructured,dense>& x, matrix<T,symmetric,sparse>& dest)
{
    dest=Pee;
    return dest*=2;
}
