@* Relative entropy. The relative entropy (also known as Kullback Leibler distance) function is
given by $x\log x/y$. The relative entropy is a measure of how much do we loose for assuming that
the distribution of some random variable is~$y$ when the true distribution is~$x$. If we knew~$x$,
then we could describe the random variable with a code with average description lenght~${\bf
entr}(x)$. If we use~$y$, however, we would need~${\bf entr}(x)+{\bf relentr}(x,y)$ to
describe the variable.
@c
/* Empty, waiting for |export| */
@ @(functional/relentr.h@>=
#ifndef __MATH_RELATIVE_ENTROPY_FUNCTIONAL__
#define __MATH_RELATIVE_ENTROPY_FUNCTIONAL__ 1.00
#include <math.h>
#include <math/functionalbase.h>
namespace math
{
    namespace functional
    {
	template<class T>@;
	class relentr : public base<T>
	{
	    @<Relative entropy functional internal variables@>@;
	public:@;
	    @<Relative entropy functional class methods@>@;
	};
    }
}
#endif
@ @<Relative entropy functional internal variables@>=
typedef base<T>* element_type;
element_type f,g;
@ @<Relative entropy functional class methods@>=
relentr(element_type new_f,element_type new_g) : f(new_f), g(new_g) @+{@+}
@ @<Relative entropy functional class methods@>=
T eval(const matrix<T,unstructured,dense>& x)
{
   T fval=f->eval(x);
   T gval=g->eval(x);
   if(gval <=0 || fval<=0) throw error::domain();
   return fval*::log(fval/gval);
}
@ The gradient of the relative entropy is given by $(1+\log (f/g))\nabla f-(f/g)\nabla g$.
@<Relative entropy functional internal...@>=
matrix<T,unstructured,dense> auxvec;
@ @<Relative entropy functional class methods@>=
matrix<T,unstructured,dense>& grad(const matrix<T,unstructured,dense>& x,
				   matrix<T,unstructured,dense>& dest)
{
   T fval=f->eval(x);
   T gval=g->eval(x);
   if(gval <=0 || fval<=0) throw error::domain();
   g->grad(x,auxvec);
   auxvec*=fval/gval;
   f->grad(x,dest);
   dest*=(::log(fval/gval)+1);
   return dest-=auxvec;
}
@ The Hessian is given by 
$(1+\log(f/g))\nabla^2f-(f/g)\nabla^2g+f(\nabla g/g-\nabla f/f)(\nabla g/g-\nabla f/f)^T$.
@<Relative entropy functional internal variables@>=
matrix<T,unstructured,dense> auxgrad;
matrix<T,symmetric,dense> auxdense;
matrix<T,symmetric,sparse> auxsparse;
@ @<Relative entropy functional class methods@>=
matrix<T,symmetric,dense> &hess(const matrix<T,unstructured,dense>&x, matrix<T,symmetric,dense>&
				dest)
{
#define auxhess auxdense
   @<|relentr| Hessian@>;
#undef auxhess
}
matrix<T,symmetric,sparse> &hess(const matrix<T,unstructured,dense>&x, matrix<T,symmetric,sparse>&
				dest)
{
#define auxhess auxsparse
   @<|relentr| Hessian@>;
#undef auxhess
}
@ @<|relentr| Hessian@>=
   T fval=f->eval(x);
   T gval=g->eval(x);
   if(gval <=0 || fval<=0) throw error::domain();
   g->grad(x,auxvec);
   f->grad(x,auxgrad);
   auxvec/=gval;
   auxgrad/=fval;
   outerp(auxvec-=auxgrad,&dest);
   dest*=fval;
   g->hess(x,auxhess);
   auxhess*=fval/gval;
   dest-=auxhess;
   f->hess(x,auxhess);
   auxhess*=(::log(fval/gval)+1);
   return dest+=auxhess;
