@** Line Searching. We begin now the definitions that will enable us to
perform various types of numerical optimization. The first task we face is the one of minimizing
a functional along a line. There are many algorithms to perform this task, and which to use will
depend heavily on the specific problem (for example, is it cheap to compute the gradient, to
evaluate the function at a point and so on).
@s linesearch int
@c
/* Empty, waiting for |export|. */
@ @(linesearchbase.h@>=
#ifndef __MATH_LINESEARCH__
#define __MATH_LINESEARCH__ 1.0
#include<math/functionalbase.h>
namespace math{
    @[namespace @!linesearch@]
    {
	template<class T>@;    
	class @[base@]
	{
	public:@;
	    virtual ~base(void)@+{@+}
	    @<Line search base class methods@>@;
	};
    }
}
#endif
@ The interface is very simple: there is one method that applies the minimization algorithm to a
given functional, starting from a given point and searching in a given direction. The syntax is
|minimize(functional,x0,dir)|. The direction does not always need to have unity norm, but you
better check out the specific algorithm to be sure that this is the case. The |minimize| method
returns the minimizing point. As a design decision, it would be beneficial in some cases (for
example in the backtracking algorithm) to include the gradient in the list of arguments, since
the gradient would probably be available from the computation of the search direction. Since we
cannot require this (for example, in some cases the gradient would have to be computed via some
expensive simulation), we provide an alternative method that takes the gradient at the starting
point as the third argument (the search direction becomes the fourth).
@<Line search base class methods@>=
virtual matrix<T,unstructured,dense>
minimize(functional::base<T>*,const matrix<T,unstructured,dense>&,const
	 matrix<T,unstructured,dense>&)=0;
virtual matrix<T,unstructured,dense>
minimize(functional::base<T>*,const matrix<T,unstructured,dense>&,const
	 matrix<T,unstructured,dense>&, const matrix<T,unstructured,dense>&)=0;
