@* Newton direction with equality constraints.
@c 
/* Empty, waiting for |export| */
@ @(searchdir/equality/newton.h@>=
#ifndef __MATH_SEARCHDIR_EQUALITY_NEWTON__
#define __MATH_SEARCHDIR_EQUALITY_NEWTON__
#include<math/math.h>
#include<math/searchdirbase.h>
#include<math/lu.h>
namespace math {
namespace searchdir {
namespace equalty {
template<class T,template<class> class storage>
class newton : public base<T>
{
    @<Newton with equality internal variables@>;
    public:
    @<Newton with equality functions@>;
};
}}}
#endif
@ Let us be wise memorywise. Unlike with the pure equality search direction, we will have to
restore the system matrix every time we compute a new direction. So that's what we do: we store
the~|A| matrix literallyi. This matrix will not change once its initialized. Next, we keep the
system matrix in a matrix~|M|.
@<Newton with equality int...@>=
matrix<T,unstructured,storage> A;
matrix<T,symmetric,storage> H;
matrix<T,unstructured,storage> M;
@ Of course, we also face the same auxiliary variable problem.
@<Newton with equality int...@>=
bool has_t;
index xrows; /* Original number of variables. */
@ @<Newton with equality f...@>=
matrix<T,unstructured,dense>&
dir(functional::base<T>* f,const matrix<T,unstructured,dense>&
	x,matrix<T,unstructured,dense>* dest)
{
    if(x.rows()!=xrows && has_t==false)
    {
	has_t=true;
    	A.resize(A.rows(),xrows+1);
	for(index i=1;i<=A.rows();++i) A.entry(i,xrows+1)=0;
    }
    if(x.rows()==xrows && has_t==true)
    {
	has_t=false;
	A.resize(A.rows(),xrows);
    }
    try 
    {
	f->grad(x,*dest);
	f->hess(x,H);
#ifdef __MATH_DEBUG__
	cout << "[math]: got Hessian.\n";
	cout << "H=\n";
	for(index i=1;i<=H.rows();++i)
	{
	    cout << "row " << i << ": ";
	    for(index j=1;j<=H.cols();++j) cout << H(i,j) << ' ';
	    cout << '\n';
	}
#endif
	dest->resize(x.rows()+A.rows(),1);
	M.resize(x.rows()+A.rows(),x.rows()+A.rows());
	for(index i=1;i<=A.rows();++i)
	{
	    dest->entry(i+x.rows(),1)=0;
	    for(index j=1;j<=A.rows();++j) M.entry(i+x.rows(),j+x.rows())=0;
	    for(index j=1;j<=A.cols();++j)
	    {
		M.entry(i+x.rows(),j)=A(i,j);
		M.entry(j,i+x.rows())=A(i,j);
	    }
	}
	for(index i=1;i<=x.rows();++i)
	    for(index j=i;j<=x.rows();++j)
	    {
		M.entry(i,j)=H(i,j);
		M.entry(j,i)=H(j,i);
	    }
#ifdef __MATH_DEBUG__
	cout << "[math]: about to get search direction.\n";
	cout << "M=\n";
	for(index i=1;i<=M.rows();++i)
	{
	    for(index j=1;j<=M.cols();++j) cout << M(i,j) << ' ';
	    cout << '\n';
	}
	cout << "dest=\n";
	for(index i=1;i<=dest->rows();++i) cout << dest->get(i,1) << ' ';
	cout << '\n';
#endif
	lu::solve(M,*dest);
	dest->resize(x.rows(),1);
#ifdef __MATH_DEBUG__
	cout << "dest=\n";
	for(index i=1;i<=dest->rows();++i) cout << dest->get(i,1) << ' ';
	cout << '\n';
#endif
    }
    catch(error::singular e) { f->grad(x,*dest); }
    return (*dest)*=-1;
}
@ Now we provide a function to set the~|A| matrix.
@<Newton with equality f...@>=
void set_A(const matrix<T,unstructured,storage>& a)
{
    has_t=false;
    xrows=a.cols();
    A=a;
}

