@** The sparse storage. Now that we have the basics defined, let us exemplify how to create
new types of matrices by defining a new storage type.
@c
/* Empty, waiting for |export| */
@ @(sparse.h@>=
#ifndef __MATH_SPARSE__
#define __MATH_SPARSE__ 1.0
@<Include files |sparse|@>@;
namespace math
{
    @<Sparse storage definition@>@;
}
#endif
@ @<Sparse storage definition@>=
template<class T>@;
class sparse
{
    @<Sparse storage internal variables@>@;
public:@;
    @<Sparse storage methods@>@;
};
@ A sparse matrix stores its elements in a map in which the key is the element index. To
simplify notation we define a new variable for the map type.
@<Sparse storage internal variables@>=
typedef map<math::pair,T,@[less<math::pair>@]> storage;
storage *elements;
@ @<Include files |sparse|@>=
#include <math/math.h> /* For |math::pair| -- otherwise unecessary. */
#include <map>
@ Now to the interesting part, that is, to the definition of methods that a matrix storage
must have. We begin by the constructors and destructor: as with the |dense| case, we define
a default constructor, a ``dimension'' constructor and a copy constructor. The interesting
part is that we don't need to initialize any data or keep any information whatsoever in the
creation phase.
@<Sparse storage methods@>=
sparse(void) @[:@] elements(0)@+ {}
sparse(const index &rows,const index &cols) @[:@] elements(0) @+
{@+
    elements=new storage;@+
}
sparse(const sparse &source) @[:@] elements(0)
{
    elements=new storage;
    elements->insert(source.elements->begin(),source.elements->end());
}
@ @<Sparse storage methods@>=
~sparse()@+
{@+
    if(elements) delete elements;@+
}
@ We have now to define how to set and get elements. When setting elements we have to
consider setting an element to zero -- if the element exists we remove it, otherwise we
don't set anything.
@<Sparse storage methods@>= @!@:sparse}{set@>
void set(const index &row,const index& col,const T& value)
{
    math::pair i(row,col);
    if(value!=T(0))
        (*elements)[i]=value;
    else if(elements->find(i)!=elements->end())
            elements->erase(i);
}
@ Getting elements is not trivial only because if the default behavior of maps is to create
non-existing entry when accessing it. Recall that if either~|row| or~|col| are zero we are
supposed to return~$0$ -- which will happen because there is no element with index~$(0,0)$ stored 
in a sparse matrix.
@<Sparse storage methods@>= @!@:sparse}{get@>
const T get(const index &row, const index &col) const
{
    math::pair i(row,col);
    if(elements->find(i)!=elements->end())
        return (*elements)[i];
    return T(0);
}
@ When resizing we let elements outside the new matrix to stay there just to make resizing fast
(very fast, just an empty method). It is assumed that you won't be resizing a sparse matrix to
very different sizes.
@<Sparse storage methods@>=@!@:sparse}{resize@>
void resize(const index &rows, const index& cols)@+{@+}
@ And we are done. The methods listed for this class are the only required methods for a
valid matrix storage type. Now that we have the dense and sparse types, however, there is
little need for other types, so I consider this work done. The only other kind of storage I
can imagine is the block-diagonal storage, but then we have some difficulties (we can have
dense and sparse block-diagonal matrices and so on).
